@@ -6,7 +6,7 @@ use strict;
 #	Public Global Variables
 #-----------------------------------------------------------------------
 use vars qw($VERSION);
-$VERSION   = '5.05';
+$VERSION   = '5.06';
 
 #=======================================================================
 sub new {
@@ -21,6 +21,7 @@ sub new {
     my (@names, @lengths, %justify);
     my $length = 0;
     if (defined $delim) {
+        $self->{DELIM} = $delim;
         for (@$format) {
             my ($name, $len) = split $delim;
             push @names, $name;
@@ -97,23 +98,47 @@ sub pack {
 }
 #=======================================================================
 sub names {
-   shift->{NAMES};
+    shift->{NAMES};
 }
 #=======================================================================
 sub length {
-   my $self = shift;
-   my $field = shift or return $self->{LENGTH};
-   $self->{LENGTHS}{$field};
+    my $self = shift;
+    my $field = shift or return $self->{LENGTH};
+    $self->{LENGTHS}{$field};
+}
+#=======================================================================
+sub dumper {
+    my $parser = shift;
+    my $start = 1;
+    my $end;
+    my $delim = $parser->{DELIM};
+    my $format = (defined $delim)
+        ? sub { join $delim, @_ }
+        : sub { sprintf("%s => %s, # %s-%s", @_) };
+    my $layout = '';
+    for my $name (@{$parser->names}) {
+        my $len = $parser->length($name);
+        $end = $start + $len - 1;
+        $layout .= $format->($name, $len, $start, $end) . "\n";
+        $start = $end + 1;
+    }
+    $layout;
 }
 #=======================================================================
-# Maybe not a clean OO way, this should be a different
-# object in a different class, but oh well...it ain't rocket science.
 sub converter {
-   my ($parser1, $parser2, $mappings, $defaults) = @_;
-   # Do the OO cargo cult dance
+    Parse::FixedLength::Converter->new(@_);
+}
+
+package Parse::FixedLength::Converter;
+
+#=======================================================================
+sub new {
+   # Do the OO cargo cult constructor dance
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = bless {}, $class;
+
+   my ($parser1, $parser2, $mappings, $defaults) = @_;
    $self->{UNPACKER} = $parser1;
    $self->{PACKER} = $parser2;
    $self->{MAP} = { reverse %$mappings };
@@ -152,29 +177,23 @@ __END__
 
 =head1 NAME
 
-Parse::FixedLength - parse a ascii string containing fixed length fields into component parts
+Parse::FixedLength - parse an ascii string containing fixed length fields into component parts
 
 =head1 SYNOPSIS
 
     use Parse::FixedLength;
     
-    my $parser = Parse::FixedLength->new([
-        first_name => 10,
-        last_name  => 10,
-        address    => 20,
-    ]);
+    $parser = Parse::FixedLength->new(\@format);
+    $parser = Parse::FixedLength->new(\@format, \%parameters);
 
-    my $data = 'Bob       Jones     1122 Main St.       ';
-    my $parsed_href = $parser->parse($data);
-    my $orig_data = $parser->pack($parsed_href);
+    $hash_ref = $parser->parse($data);
+    $data = $parser->pack($hash_ref);
 
-    or:
+    $converter = $parser1->converter($parser2);
+    $converter = $parser1->converter($parser2, \%mappings);
+    $converter = $parser1->converter($parser2, \%mappings, \%defaults);
 
-    my $parser = Parse::FixedLength->new([qw(
-        first_name:10
-        last_name:10
-        address:20
-    )], {delim=>":"});
+    $data_out = $converter->convert($data_in);
 
 =cut
 
@@ -191,15 +210,31 @@ a string into its fixed-length components.
 
 =item new() 
 
- $parser = Parser::FixedLength->new($aref_format, $href_parameters)
+ $parser = Parser::FixedLength->new(\@format)
+ $parser = Parser::FixedLength->new(\@format, \%parameters)
 
 This method takes an array reference of field names and
 lengths as either alternating elements, or delimited args in the
-same field.
+same field, e.g.:
+
+    my $parser = Parse::FixedLength->new([
+        first_name => 10,
+        last_name  => 10,
+        address    => 20,
+    ]);
+
+    or:
+
+    my $parser = Parse::FixedLength->new([qw(
+        first_name:10
+        last_name:10
+        address:20
+    )], {delim=>":"});
 
 To right justify a field (during the 'pack' method), an "R" may
 be appended to the length of the field along with (optionally)
-the character to pad the string with. This is somewhat inefficient,
+the character to pad the string with (if no character follows the
+"R", then a space is assumed). This is somewhat inefficient,
 so its only recommended if actually necessary to preserve the format
 during operations such as math or converting format lengths. If its
 not needed but you'd like to specify it anyway for documentation
@@ -209,7 +244,8 @@ the data in the hash ref argument.
 An optional hash ref may also be supplied which may contain the following:
 
  delim - The delimiter used to separate the name and length in
-         the format array.
+         the format array. If another delimiter follows the length
+         then any 'extra' fields are ignored.
 
  spaces - If true, preserve trailing spaces during parse.
 
@@ -220,7 +256,8 @@ An optional hash ref may also be supplied which may contain the following:
 
 =item parse() 
 
- $href_parsed_data = $parser->parse($string_to_parse)
+ $hash_ref = $parser->parse($string)
+ @ary      = $parser->parse($string)
 
 This function takes a string and returns the results of
 fixed length parsing as a hash reference of field names and
@@ -228,26 +265,58 @@ values if called in scalar context, or just a list of the
 values if called in list context.
 
 =item pack
- $packed_str = $parser->pack($href_data_to_pack);
+
+ $data = $parser->pack(\%data_to_pack);
 
 This function takes a hash reference of field names and values and
 returns a fixed length format output string.
 
 =item names()
 
- $aref_names = $parser->names;
+ $ary_ref = $parser->names;
 
 Return an ordered arrayref of the field names.
 
 =item length()
 
  $tot_length   = $parser->length;
- $field_length = $parser->length($field_name);
+ $field_length = $parser->length($name);
 
 Returns the total length of all the fields, or of just one field name.
+E.g.:
+
+ while (read FH, $data, $parser->length) {
+  $parser->parse($data);
+  ...
+ }
+
+=item dumper()
+
+ $parser->dumper;
+
+Returns the parser's format layout information in a format suitable
+for cutting and pasting into the format array argument of a
+Parse::FixedFormat->new() call, and includes the start and end positions
+of all the fields (starting with position 1). E.g.:
+
+ # Assume the parser is from the ones defined in the new() example:
+ print $parser->dumper;
+
+ produces:
+ first_name => 10, # 1-10
+ last_name => 10, # 11-20
+ address => 20, # 21-40 
+
+ or:
+
+ first_name:10:1:10
+ last_name:10:11:20
+ address:20:21:40
 
 =item converter()
 
+ $converter = $parser1->converter($parser2);
+ $converter = $parser1->converter($parser2, \%mapping);
  $converter = $parser1->converter($parser2, \%mapping, \%defaults);
 
 Returns a format converting object. $parser1 is the parsing object
@@ -255,10 +324,11 @@ to convert from, $parser2 is the parsing object to convert to.
 
 By default, common field names will be mapped from one format to the other.
 Fields with different names can be mapped from the first format to the
-other (or you can override the default) using the second argument
-(See examples).
+other (or you can override the default) using the second argument.
+The keys are the source field names and the corresponding values are
+the target field names.
 
-And defaults for any field in the target format can be supplied
+Defaults for any field in the target format can be supplied
 using the third argument, where the keys are the field names of
 the target format, and the value can be a scalar constant, or a
 subroutine reference where the first argument is simply the mapped
@@ -266,13 +336,14 @@ value (or the empty string if there was no mapping), and the
 second argument is the entire hash reference that results from parsing
 the data with the 'from' parser object. E.g. if you were mapping
 from a separate 'zip' and 'plus_4' field to a 'zip_plus_4' field,
-you could supply in the default hash ref the following:
+you could supply as one of the key/value pairs in the 'defaults' hash
+ref the following:
 
  zip => sub { shift() . $$_[1]{plus_4} }
 
 =item convert()
 
- $formatted_in_str = $converter->convert($formatted_out_str);
+ $data_out = $converter->convert($data_in);
 
 Converts a string from one fixed length format to another.
 
@@ -302,11 +373,11 @@ Converts a string from one fixed length format to another.
         print $parser->pack($data), "\n";
     }
     __DATA__
-    BOB       JONES        24
-    JOHN      SMITH         5
-    JANE      DOE           7
+    BOB       JONES     00024
+    JOHN      SMITH     00005
+    JANE      DOE       00007
 
-    A alternate way if we're converting formats:
+    Another way if we're converting formats:
 
     my $parser1 = Parse::FixedLength->new([
         first_name => 10,
@@ -314,13 +385,14 @@ Converts a string from one fixed length format to another.
         widgets_this_month => '5R0',
     ]);
 
-    my $parser2 = Parse::FixedLength->new([
-        seq_id     => 10,
-        first_name => 10,
-        last_name  => 10,
-        country    =>  3,
-        widgets_this_year => '10R0',
-    ]);
+    # Use delim option just for example
+    my $parser2 = Parse::FixedLength->new([qw(
+        seq_id:10
+        first_name:10
+        last_name:10
+        country:3
+        widgets_this_year:10R0
+    )]);
 
     my $converter = $parser1->converter($parser2, {
         widgets_this_month => widgets_this_year,
@@ -333,8 +405,6 @@ Converts a string from one fixed length format to another.
     });
     
 
-    # Do a simple name casing of names
-    # and print widgets projected for the year for each person
     while (<DATA>) {
         warn "No record terminator found!\n" unless chomp;
         warn "Short Record!\n" unless $parser1->length == length;
